<?php declare(strict_types=1);

namespace App\Controller;

use App\Entity\Film;
use App\Form\FilmType;
use App\Traits\DeleteFormTrait;
use App\Traits\FormControllerTrait;
use Doctrine\Common\Collections\Criteria;
use Doctrine\ORM\EntityManagerInterface;
use LogicException;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\{RedirectResponse, Request, Response};
use Symfony\Component\Routing\Annotation\Route;

/**
 * Film controller.
 */
#[Route(path: 'film')]
class FilmController extends AbstractController
{
    use FormControllerTrait;
	use DeleteFormTrait;

    protected const ENTITY = Film::class;
	protected const TEMPLATE_PATH = 'film/';
	protected const ROUTE_PREFIX = 'film_';
    protected const FORM   = FilmType::class;

    public function __construct(private readonly EntityManagerInterface $entityManager)
    {
    }

    /**
     * Lists all film entities.
     */
    #[Route(path: '/', name: 'film_index', methods: ['GET'])]
    public function indexAction(): Response
    {
        $repo     = $this->entityManager->getRepository(self::ENTITY);
        $criteria = Criteria::create()
            ->where(Criteria::expr()->gt('rollsInCamera', 0))
            ->orderBy([
                'filmFormat'    => Criteria::ASC,
                'brand'         => Criteria::ASC,
                'rollsInCamera' => Criteria::DESC,
                'productLine'   => Criteria::ASC,
            ]);
        $inCamera    = $repo->matching($criteria);
        $notInCamera = $repo->findBy([
            'rollsInCamera' => 0,
        ], [
            'brand'       => 'ASC',
            'productLine' => 'ASC',
            'filmFormat'  => 'ASC',
        ]);

        return $this->render('film/index.html.twig', [
            'in_camera' => $inCamera,
            'films'     => $notInCamera,
        ]);
    }

    /**
     * Creates a new film entity.
     */
    #[Route(path: '/new', name: 'film_new', methods: ['GET', 'POST'])]
    public function newAction(Request $request): RedirectResponse|Response
    {
        return $this->itemCreate($request, 'film');
    }

    /**
     * Finds and displays a film entity.
     */
    #[Route(path: '/{id}', name: 'film_show', methods: ['GET'])]
    public function showAction(Film $film): Response
    {
        return $this->itemView($film, 'film');
    }

    /**
     * Displays a form to edit an existing film entity.
     *
     * @throws LogicException
     */
    #[Route(path: '/{id}/edit', name: 'film_edit', methods: ['GET', 'POST'])]
    public function editAction(Request $request, Film $film): RedirectResponse|Response
    {
        return $this->itemUpdate($request, $film, 'film');
    }

    /**
     * Deletes a film entity.
     *
     * @throws LogicException
     */
    #[Route(path: '/{id}', name: 'film_delete', methods: ['POST','DELETE'])]
    public function deleteAction(Request $request, Film $film): RedirectResponse
    {
        return $this->itemDelete($request, $film);
    }
}
